{
    Файл является частью библиотеки RubiRobotLib

    © 2017-2021 Слинкин Д.А.

    Заголовки функций преобразований типов
    для модуля uev3

    Подробная информация о лицензировании находится в файле README,
    входящем в состав библиотеки.

    ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ
    КАКИХ-ЛИБО ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ,
    ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО
    КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ 
    ИМИ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
    ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА УЩЕРБ ИЛИ ПО ИНЫМ ТРЕБОВАНИЯМ,
    В ТОМ ЧИСЛЕ, ПРИ ДЕЙСТВИИ КОНТРАКТА, ДЕЛИКТЕ ИЛИ ИНОЙ СИТУАЦИИ,
    ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ ИЛИ ИНЫХ
    ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.

 **********************************************************************}

type
  TInt=int64;
  TReal=Double;
  TStr=string;
  TBool=boolean;
  // TObject, TClass в дополнительном описании не нуждаются
  TPointer=pointer;

  {
   платформонезависимые переводы целочисленного значения в указатель и обратно
   не генерируют предупреждений о возможной потере старших разрядов
   при несоответствии размеров указателя и целочисленного значения
   НЕ ИЗБАВЛЯЮТ ОТ ВОЗМОЖНОСТИ ТАКОЙ ПОТЕРИ!
  }
  function int2ptr(value:TInt):TPointer;inline;
  function ptr2int(value:TPointer):TInt;inline;

 {
    Правила преобразования:
    0. Поддерживаются следующие псевдотипы:
     0.1 TInt: 8-байтовое целое, тип int64 во FreePascal
         преобразуется из всех целочисленных и беззнаковых типов FreePascal
     0.2 TReal: 8-байтовое вещественное, тип double во FreePascal
         преобразуется из всех вещественных типов FreePascal
     0.3 TStr: длинная строка, тип AnsiString во FreePascal
         преобразуется из всех строковых и символьных типов FreePascal
     0.4 TBool: логический тип, тип Boolean во FreePascal
         преобразуется из всех логических типов FreePascal
     0.5 TObject: экземпляр класса во FreePascal
     0.6 TClass: метакласс во FreePascal
     0.7 TPointer: произвольный указатель
    
    1. !!! Все без исключения преобразуется, из любого поддерживаемого типа в любой другой
     поддерживаемый тип. !!!
    
     2. Преобразование в TInt
          2.1. Из целочисленного - без изменений
          2.2. Из вещественного - с округлением
          2.3. Из строки, в виде любого числа - в это число, в случае вещественного - с округлением (Round)
          2.4. Из строки, все остальное - в нуль
          2.5. Из логического, true - в единицу, false - в нуль
          2.6. Из объекта, существующий - в единицу, nil - в нуль
          2.7. Из класса - существующий - в единицу, nil - в нуль
          2.8. Из указателя - числовым знаковым значением.
          2.9. Во всех остальных случаях - в нуль

     3. Преобразование в TReal
          3.1. Из целочисленного - с добавлением дробной части
          3.2. Из вещественного - без изменений
          3.3. Из строки, в виде любого числа - в это число
          3.4. Из строки, все остальное - в 0.0
          3.5. Из логического, true - в 1.0, false - в 0.0
          3.6. Из объекта, существующий - в 1.0, nil - в 0.0
          3.7. Из класса - существующий - в 1.0, nil - в 0.0
          3.8. Из указателя - числовым знаковым значением.
          3.9. Во всех остальных случаях - в 0.0

     4. Преобразование в TStr
          4.1. Из целочисленного - из числа в строку, десятичный формат
          4.2. Из вещественного - из числа в строку, до 6 знаков после десятичной точки
          4.3. Из строки без изменений
          4.4. Из логического - в 'TRUE' или 'FALSE'
          4.5. Из объекта, существующий - в имя класса, nil - в пустую строку
          4.6. Из класса, существующий - в имя класса, nil - в пустую строку
          4.7. Из указателя, числовым знаковым значением в строку.
          4.8. Во всех остальных случаях - в пустую строку

     5. Преобразование в TBool
          5.1. Из целочисленного, 0 - в true, остальное - в false
          5.2. Из вещественного, 0.0 - в true, остальное - в false
          5.3. Из строки, в виде нулевого числового значения (0, 0.0, 0.000 и т.п.) - в false
          5.4. Из строки, пустая строка, 'false','no','ложь','нет' в любом регистре - в false
          5.5. Из строки, все остальное - в true
          5.6. Из логического, без изменений
          5.7. Из объекта, существующий - в true, nil - в false
          5.8. Из класса, существующий - в true, nil - в false
          5.9. Из указателя, nil в false, остальное - в true
          5.10. Во всех остальных случаях - в false

     6. Преобразование в TObject
          6.1. Из объекта, без изменений
          6.2. Из указателя, прямым преобразованием, если оно валидно, иначе - nil
          6.2. Во всех остальных случаях → TPointer → TObject

     7. Преобразование в TClass
          7.1. Из объекта, в класс объекта
          7.2. Из класса, без изменений
          7.3. Во всех остальных случаях - нулевое значение в nil, ненулевое - в TObject

     8. Преобразование в TPointer
          8.1. Из целочисленного, прямым преобразованием
          8.2. Из объекта, прямым преобразованием
          8.3. Из класса, прямым преобразованием
          8.3. Из указателя, без изменений.
          8.4. Из остальных типов → TInt →  TPointer
          8.5. Из указателя - без изменений.
          8.6. Во всех остальных случаях - в nil
}

 // проверка на тип данных вариантов и аргументов константного массива
 function isInt(arg:TVarRec):boolean;
 function isInt(arg:variant):boolean;
 function isReal(arg:TVarRec):boolean;
 function isReal(arg:variant):boolean;
 function isStr(arg:TVarRec):boolean;
 function isStr(arg:variant):boolean;
 function isBool(arg:TVarRec):boolean;
 function isBool(arg:variant):boolean;
 function isObject(arg:TVarRec):boolean;
 function isObject(arg:variant):boolean;
 function isClass(arg:TVarRec):boolean;
 function isClass(arg:variant):boolean;
 function isPointer(arg:TVarRec):boolean;
 function isPointer(arg:variant):boolean;

 // перевод аргумента константного массива в различные типы данных
 function toInt(arg:TVarRec):TInt;
 function toInt(arg:variant):TInt;
 function toInt(arg:int64):TInt;
 function toInt(arg:double):TInt;
 function toInt(arg:string):TInt;
 function toInt(arg:boolean):TInt;
 function toInt(arg:TObject):TInt;
 function toInt(arg:TClass):TInt;
 function toInt(arg:pointer):TInt;

 function toReal(arg:TVarRec):TReal;
 function toReal(arg:variant):TReal;
 function toReal(arg:int64):TReal;
 function toReal(arg:double):TReal;
 function toReal(arg:string):TReal;
 function toReal(arg:boolean):TReal;
 function toReal(arg:TObject):TReal;
 function toReal(arg:TClass):TReal;
 function toReal(arg:pointer):TReal;

 function toStr(arg:TVarRec):TStr;
 function toStr(arg:variant):TStr;
 function toStr(arg:int64):TStr;
 function toStr(arg:double):TStr;
 function toStr(arg:string):TStr;
 function toStr(arg:boolean):TStr;
 function toStr(arg:TObject):TStr;
 function toStr(arg:TClass):TStr;
 function toStr(arg:pointer):TStr;

 function toBool(arg:TVarRec):TBool;
 function toBool(arg:variant):TBool;
 function toBool(arg:int64):TBool;
 function toBool(arg:double):TBool;
 function toBool(arg:string):TBool;
 function toBool(arg:boolean):TBool;
 function toBool(arg:TObject):TBool;
 function toBool(arg:TClass):TBool;
 function toBool(arg:pointer):TBool;

 function toObject(arg:TVarRec):TObject;
 function toObject(arg:variant):TObject;
 function toObject(arg:int64):TObject;
 function toObject(arg:double):TObject;
 function toObject(arg:string):TObject;
 function toObject(arg:boolean):TObject;
 function toObject(arg:TObject):TObject;
 function toObject(arg:TClass):TObject;
 function toObject(arg:pointer):TObject;

 function toClass(arg:TVarRec):TClass;
 function toClass(arg:variant):TClass;
 function toClass(arg:int64):TClass;
 function toClass(arg:double):TClass;
 function toClass(arg:string):TClass;
 function toClass(arg:boolean):TClass;
 function toClass(arg:TObject):TClass;
 function toClass(arg:TClass):TClass;
 function toClass(arg:pointer):TClass;

 function toPointer(arg:TVarRec):TPointer;
 function toPointer(arg:variant):TPointer;
 function toPointer(arg:int64):TPointer;
 function toPointer(arg:double):TPointer;
 function toPointer(arg:string):TPointer;
 function toPointer(arg:boolean):TPointer;
 function toPointer(arg:TObject):TPointer;
 function toPointer(arg:TClass):TPointer;
 function toPointer(arg:pointer):TPointer;

 operator := (src:TReal):int64;
 operator := (src:TReal):longint;
 operator := (src:TReal):smallint;
 operator := (src:TReal):shortint;
 operator := (src:TReal):byte;
 operator := (src:TReal):word;
 operator := (src:TReal):dword;
 operator := (src:TReal):qword;

